import { version, markRaw, ref, computed, watch, onBeforeUnmount, nextTick, inject, toRaw, reactive, toRef, getCurrentInstance, set, del, provide } from 'vue-demi';

var _interface = /*#__PURE__*/Object.freeze({
    __proto__: null
});

/** 判断是否是 2.7.* 版本, 监听生命周期需对该版本做处理 */
const IS_COMPOSITION_VERSION = version.slice(0, 3) === '2.7';
/** 容器注入的 key */
const provideKey = 'condition-wrapper';
function defineProvideValue(option) {
  return option;
}
function defineCommonMethod(option) {
  return option;
}

/**
 * 空值转为提供的默认值
 * @param {*} val 值为空时转为默认值
 * @param {*} defaultVal
 */
function emptyToValue(val, defaultVal) {
  // 数组不应该置为空值, 影响到组件内部逻辑
  // if (Array.isArray(val)) return val.filter(Boolean).length ? val : defaultVal;
  return typeof val === 'number' ? val : val || defaultVal;
}
/** 判断两个值是否相等, 排除掉空值 */
function isEqualExcludeEmptyValue(x, y) {
  return isEmptyValue(x) && isEmptyValue(y) || isEqual(x, y);
}
/** 判断是否是空值(null, undefined, '') */
function isEmptyValue(val) {
  return val === undefined || val === '' || val === null;
}
/** 判断两个值是否一致 */
function isEqual(x, y) {
  if (Object.is(x, y)) return true;
  if (x instanceof Date && y instanceof Date) {
    return x.getTime() === y.getTime();
  }
  if (x instanceof RegExp && y instanceof RegExp) {
    return x.toString() === y.toString();
  }
  if (typeof x !== 'object' || x === null || typeof y !== 'object' || y === null) {
    return false;
  }
  const keysX = Reflect.ownKeys(x);
  const keysY = Reflect.ownKeys(y);
  if (keysX.length !== keysY.length) return false;
  for (let i = 0; i < keysX.length; i++) {
    if (!Reflect.has(y, keysX[i])) return false;
    if (!isEqual(x[keysX[i]], y[keysX[i]])) return false;
  }
  return true;
}
/** 判断是否是原始类型(number , string , boolean , symbol, bigint, undefined, null) */
function isPrimitive(value) {
  return value === undefined || value === null || typeof value !== 'object' && typeof value !== 'function';
}
/** 拷贝 */
function clone(obj, deep) {
  if (isPrimitive(obj)) return obj;
  if (typeof obj === 'function') return obj.bind({});
  const newObj = new obj.constructor();
  Object.getOwnPropertyNames(obj).forEach(prop => {
    newObj[prop] = deep ? clone(obj[prop]) : obj[prop];
  });
  return newObj;
}
/**
 * 获取指定层级的父级(包括自身)
 * @param {Record<string, any>[]} data 数据源
 * @param {(item) => boolean} cb 当前数据项是否匹配
 */
function getChained(data, cb, childrenKey = 'children') {
  for (const item of data) {
    if (cb(item)) {
      return [item];
    } else if (item[childrenKey]?.length) {
      const r = getChained(item[childrenKey], cb);
      if (r.length) {
        r.unshift(item);
        return r;
      }
    }
  }
  return [];
}
/**
 * 获取渲染节点
 * @param {string | object | Function} node 需渲染元素
 */
function getNode(node, ...args) {
  // 直接抛出 null, template 中会报错
  if (!node) return null;
  return typeof node === 'function' ? node(...args) : typeof node === 'string' ? node : markRaw(node);
}
/**
 * 通过字符串路径获取值
 * @example get(person, 'friends[0].name')
 */
function get(value, path, defaultValue) {
  const segments = path.split(/[.[\]]/);
  let current = value;
  for (const key of segments) {
    if (current === null) return defaultValue;
    if (current === undefined) return defaultValue;
    const dequoted = key.replace(/['"]/g, '');
    if (dequoted.trim() === '') continue;
    current = current[dequoted];
  }
  if (current === undefined) return defaultValue;
  return current;
}
/**
 * 判断对象是否存在指定属性
 * @example hasOwn({}, 'a')
 */
function hasOwn(obj, key) {
  // eslint-disable-next-line no-prototype-builtins
  return Object.hasOwn ? Object.hasOwn(obj, key) : obj.hasOwnProperty(key);
}

/** 获取条件的初始值 */
function useInitialValue(props) {
  const setValue = ref();
  const a = computed({
    set(value) {
      setValue.value = value;
    },
    get() {
      return setValue.value === undefined ? props.defaultValue !== undefined ? typeof props.defaultValue === 'function' ? props.defaultValue(props.query, props.backfill) : props.defaultValue : undefined : setValue.value;
    }
  });
  return a;
}
/**
 * 获取当前组件的显示和隐藏状态
 */
function useDisplay(props, option) {
  const insetDisabled = ref(typeof props.disabled === 'boolean' ? props.disabled : false);
  const insetHide = ref(typeof props.hide === 'boolean' ? props.hide : false);
  const getOption = () => ({
    query: props.query,
    backfill: props.backfill,
    option
  });
  const cb = () => {
    if (typeof props.hide === 'function') {
      const currentValue = insetHide.value;
      const newValue = props.hide(getOption());
      if (currentValue !== newValue) {
        insetHide.value = props.hide(getOption());
      }
    } else if (typeof props.disabled === 'function') {
      const currentValue = insetDisabled.value;
      const newValue = props.disabled(getOption());
      if (currentValue !== newValue) {
        insetDisabled.value = props.disabled(getOption());
      }
    }
  };
  let listeners = [watch(() => props.query, cb, {
    immediate: true,
    deep: true
  }), watch(() => [props.disabled, props.hide], (val, val2) => {
    if (val[0] !== val2[0]) {
      insetDisabled.value = typeof val[0] === 'boolean' ? val[0] : false;
      typeof val[0] === 'function' && (true);
    }
    if (val[1] !== val2[1]) {
      insetHide.value = typeof val[1] === 'boolean' ? val[1] : false;
      typeof val[1] === 'function' && (true);
    }
    cb();
  })];
  // eslint-disable-next-line no-sequences
  onBeforeUnmount(() => (listeners.forEach(o => o()), listeners = []));
  return {
    insetDisabled,
    insetHide
  };
}
/**
 * 转换当前事件循环内更新标识
 * @param {boolean} initialValue 初始状态
 */
function useDisableInCurrentCycle(initialValue = true) {
  /** 是否允许改变 */
  const flag = ref(initialValue);
  /** 禁止改变 */
  const updateFlag = () => {
    flag.value = !initialValue;
    nextTick(() => {
      flag.value = initialValue;
    });
  };
  return {
    flag,
    updateFlag
  };
}

/** 封装扁平组件必备的信息 */
function usePlain(props) {
  const unwatchs = [];
  /** 容器注入值 */
  const wrapper = inject(provideKey);
  /** 初始值(重置时回填的值) */
  const initialValue = useInitialValue(props);
  /** 初始是否存在回填值 */
  const initialBackfillValue = props.fields?.length
  // 防止回填值不存在时产生一个空数组(undefined[])
  ? props.fields.map(key => props.query[key]).filter(Boolean) : props.query[props.field];
  /** 当前选中值 */
  const checked = ref(initialBackfillValue || (props.defaultValue !== undefined ? clone(initialValue.value) : undefined));
  /** 远程获取的数据源 */
  const remoteOption = ref([]);
  /** 渲染的数据源(远程数据源 > 本地数据源) */
  const finalOption = computed(() => remoteOption.value.length ? remoteOption.value : props.options);
  unwatchs.push(watch(finalOption, value => wrapper && (wrapper.options[props.field] = value), {
    immediate: true
  }));
  const getQuery = () => {
    if (props.customGetQuery) return props.customGetQuery(checked.value, emptyToValue, props);
    const _checked = clone(checked.value);
    return props.multiple && props.fields ? props.fields.reduce(
    // eslint-disable-next-line no-sequences
    (p, k, i) => (p[k] = emptyToValue(_checked?.[i], props.emptyValue), p), {}) : {
      [props.field]: emptyToValue(_checked, props.emptyValue)
    };
  };
  const option = defineCommonMethod({
    reset() {
      const {
        multiple
      } = props;
      checked.value = props.resetToInitialValue && clone(initialValue.value) || (multiple ? [] : '');
    },
    resetField(allowEmptyValue) {
      const r = clone(initialValue.value);
      const isEmpty = isEmptyValue(r);
      allowEmptyValue ? checked.value = r || props.multiple ? [] : '' : isEmpty || (checked.value = r);
      return !!allowEmptyValue || !isEmpty;
    },
    updateWrapperQuery() {
      wrapper && Object.entries(getQuery()).forEach(([k, v]) => wrapper.updateQueryValue(k, v, props.field));
    },
    get validator() {
      return props.validator;
    },
    getQuery
  });
  wrapper?.register(option);
  const {
    insetDisabled,
    insetHide
  } = useDisplay(props, option);
  /** 不存在回填值且存在默认值时更新父级中的值 */
  if (initialBackfillValue === undefined && props.defaultValue !== undefined) {
    option.updateWrapperQuery();
  }
  onBeforeUnmount(() => unwatchs.forEach(v => v()));
  // 提交字段发生改变时, 删除原有字段并更新最新值
  unwatchs.push(watch(() => props.field, (val, oldVal) => {
    val !== oldVal && wrapper?.removeUnreferencedField(oldVal);
    option.updateWrapperQuery();
  }));
  // 实时值发生变化时触发更新 - 共享同一个字段
  unwatchs.push(watch([() => props.fields || props.field, () => props.fields ? props.fields.map(k => props.query[k]).filter(Boolean) : props.query[props.field]], ([_field, val], [__field]) => {
    const _val = props.backfillToValue(val, _field, props.query);
    if (checked.value === _val || _field.toString() !== __field.toString() || isEqualExcludeEmptyValue(_val, checked.value)) {
      return;
    }
    // 实时值改变先判断值是否为空
    // 为空时, 存在初始值且允许重置为初始值时, 用初始值替代, 且通知上层组件
    // 否则直接更新值即可
    checked.value !== _val && (isEmptyValue(_val) && props.resetToInitialValue && !isEmptyValue(initialValue.value) ? change(_val) : checked.value = _val);
  }, {
    flush: 'sync'
  }));
  /** 是否允许依赖变动时, 重置值(外部通过 search, change 主动改变值时, 内部应取消重置) */
  const {
    flag: allowDependChangeValue,
    updateFlag: updateAllowDependChangeValue
  } = useDisableInCurrentCycle(true);
  // 存在依赖项
  unwatchs.push(watch([() => props.depend, () => props.dependFields, () => props.dependFields && [].concat(props.dependFields).map(k => get(props.query, k))], ([_depend, _dependFields], [__depend, __dependFields]) => {
    // 是否启用依赖, 相同时启用才走后续逻辑, 不同时直接走后续逻辑
    if (_depend === __depend && !_depend) return;
    getOption('depend');
    // 类空值时, 不触发 change 事件
    // 防止表单类监测值发生改变时触发校验
    // 或内部不允许重置时直接返回
    if (!props.resetByDependValueChange || isEmptyValue(checked.value) || !allowDependChangeValue.value) return;
    change(isEmptyValue(initialValue.value) ? props.multiple ? [] : '' : clone(initialValue.value));
  }, {
    flush: 'sync',
    ...props.dependWatchOption
  }));
  // 存在选项变动依赖项时
  unwatchs.push(watch([() => props.optionsDepend, () => wrapper && (props.optionsDependFields || props.dependFields) && [].concat(props.optionsDependFields || props.dependFields).map(k => wrapper.options[k])], ([_depend], [__depend]) => {
    // 是否启用依赖, 相同时启用才走后续逻辑, 不同时直接走后续逻辑
    if (_depend === __depend && !_depend) return;
    getOption('depend');
  }));
  // 监听 getOptions 选项
  unwatchs.push(watch(() => props.getOptions, getOption.bind(null, 'initial')));
  nextTick(getOption.bind(null, 'initial'));
  /** 获取数据源发生变化事件 */
  function getOption(trigger) {
    props.getOptions?.(data => {
      const _checked = checked.value;
      // 重置 checked, 防止增加 option 后, select 值没更新的问题
      checked.value = undefined;
      remoteOption.value = data || [];
      checked.value = _checked;
    }, props.query || {}, {
      trigger,
      options: toRaw(wrapper?.options) || {},
      changeDefaultValue(value) {
        initialValue.value = value;
        return this;
      },
      change(value, isInitial) {
        isInitial && (initialValue.value = value);
        updateAllowDependChangeValue();
        change(value);
        return this;
      },
      search(value, isInitial) {
        isInitial && (initialValue.value = value);
        updateAllowDependChangeValue();
        updateCheckedValue(value);
        wrapper?.search();
        return this;
      }
    });
  }
  /**
   * 更新选中值(父级也同步更改)
   * @param {*} value 待更改的值
   */
  function updateCheckedValue(value) {
    // updateWrapperQuery 必须要执行, 防止在 custom-render 中改变了
    // checked.value 深层次内的值, 由于引用相同导致父级未更新的情况
    if (value !== checked.value) {
      // #fix 先判断值是否为空
      // 为空时, 存在初始值且允许重置为初始值时, 用初始值替代, 且通知上层组件
      // 否则直接更新值即可
      checked.value = isEmptyValue(value) && props.resetToInitialValue && !isEmptyValue(initialValue.value) ? clone(initialValue.value) : value;
    }
    option.updateWrapperQuery();
  }
  /**
   * 更新选中值并触发内部搜索事件
   * @param {*} value 待更改的值
   */
  function change(value) {
    updateCheckedValue(value);
    wrapper?.insetSearch();
  }
  return {
    wrapper,
    option,
    checked,
    getQuery,
    insetDisabled,
    insetHide,
    finalOption,
    updateCheckedValue,
    change,
    reset: option.reset
  };
}

/** 公共 props */
const commonProps = {
  /** 提交的字段 */
  field: {
    type: String,
    required: true
  },
  /** 是否多选 */
  multiple: {
    type: Boolean,
    default: undefined
  },
  /** 当前条件对象 - 实时变化 */
  query: {
    type: Object,
    required: true
  },
  /** 回填值的对象 - 非实时变化 */
  backfill: {
    type: Object
  },
  /** 禁用状态 */
  disabled: {
    type: [Boolean, Function]
  },
  /** 是否隐藏 -> 如果是函数, 需传递依赖项, 可根据依赖项动态隐藏 */
  hide: {
    type: [Boolean, Function]
  },
  /** 是否依赖其它字段 */
  depend: {
    type: Boolean
  },
  /** 依赖字段发生变化后是否重置值 */
  resetByDependValueChange: {
    type: Boolean,
    default: true
  },
  /** 依赖字段 */
  dependFields: {
    type: [String, Array]
  },
  /** 依赖字段监听选项 */
  dependWatchOption: {
    type: [Object]
  },
  /** 是否依赖其它字段的数据源 - 数据发生变动时触发 getOptions */
  optionsDepend: {
    type: Boolean
  },
  /** 数据源依赖字段 - 不传取 dependFields */
  optionsDependFields: {
    type: [String, Array]
  },
  /** 重置时是否置为初始值 */
  resetToInitialValue: {
    type: [Boolean]
  },
  /** 空置时提交的值 - 默认置为 undefined */
  emptyValue: {
    type: [String, Number, Boolean, null, undefined],
    default: undefined
  },
  /** 校验函数, 返回字符串不通过, 会触发提示 - 提交时触发 */
  validator: {
    type: [Function]
  },
  /** 自定义返回字段 */
  customGetQuery: {
    type: Function
  },
  /** 设置默认值 */
  defaultValue: {
    type: [String, Number, Boolean, Function, Array, Object],
    default: undefined
  }
};

/** 扁平条件类 props */
const plainProps = {
  ...commonProps,
  /** 字段集(多选时, 每个下标对应的字段可能不一样) */
  fields: {
    type: [Array]
  },
  /** 多字段时转换成选中值 */
  backfillToValue: {
    type: Function,
    default: v => v
  },
  /** 数据源 */
  options: {
    type: Array,
    default: () => []
  },
  /** 动态获取数据源 */
  getOptions: {
    type: Function
  }
};

/**
 * @file 目前该组件尚未被使用
 */
/** 封装 tree 组件必备的信息 */
function useTree(props) {
  /** 容器注入值 */
  const wrapper = inject(provideKey);
  /** 初始值(重置时回填的值) */
  const initialValue = useInitialValue(props);
  /** 当前选中值 */
  const checked = ref();
  /** 远程获取的数据源 */
  const remoteOption = ref([]);
  /** 渲染的数据源(远程数据源 > 本地数据源) */
  const finalOption = computed(() => remoteOption.value.length ? remoteOption.value : props.options);
  /** 字段集合 */
  const fieldObj = computed(() => props.fields
  // eslint-disable-next-line no-sequences
  ? props.fields.reduce((p, v) => (p[v] = props.emptyValue, p), {}) : {
    [props.field]: props.emptyValue
  });
  /** 获取当前条件所拥有的值 */
  const getQuery = () => {
    // 未初始化且不存在默认值时不返回查询值
    if (!sourceIsInit.value && !initialValue.value) return {};
    let _checked = clone(checked.value);
    const {
      ...result
    } = fieldObj.value;
    if (props.customGetQuery) return props.customGetQuery(_checked, emptyToValue, props);
    if (!_checked) return result;
    if (!props.emitPath) {
      _checked = props.multiple ? _checked.map(v => v?.slice(-1)[0]) : _checked.slice(-1)[0];
    }
    if (Array.isArray(_checked)) {
      props.fields ? _checked.forEach((o, i) => {
        result[props.fields[i]] = emptyToValue(o, props.emptyValue);
      }) : result[props.field] = _checked;
    } else {
      result[props.fields?.[0] || props.field] = emptyToValue(_checked, props.emptyValue);
    }
    return result;
  };
  // 防止触发搜索时, query 产生变化内部重复赋值
  const {
    flag: realtimeFlag,
    updateFlag: updateRealtimeFlag
  } = useDisableInCurrentCycle();
  // 防止触发搜索时, `backfill` 产生变化内部重复赋值
  const {
    flag: backfillFlag,
    updateFlag: updateBackfillFlag
  } = useDisableInCurrentCycle();
  /** 需暴露给父级操作 */
  const option = defineCommonMethod({
    reset() {
      updateRealtimeFlag();
      updateBackfillFlag();
      checked.value = props.resetToInitialValue && initialValue.value?.slice() || [];
      return this;
    },
    resetField(allowEmptyValue) {
      const r = initialValue.value?.slice();
      const isEmpty = isEmptyValue(r);
      allowEmptyValue ? checked.value = r || [] : isEmpty || (checked.value = r);
      return !!allowEmptyValue || !isEmpty;
    },
    get validator() {
      return props.validator;
    },
    updateWrapperQuery() {
      updateRealtimeFlag();
      wrapper && Object.entries(getQuery()).forEach(([k, v]) => wrapper.updateQueryValue(k, v, props.field));
    },
    getQuery
  });
  wrapper?.register(option);
  const {
    insetDisabled,
    insetHide
  } = useDisplay(props, option);
  const unwatchs = [];
  onBeforeUnmount(() => unwatchs.forEach(v => v()));
  const sourceIsInit = ref(typeof props.getOptions !== 'function' || !!props.fields?.length);
  watch(sourceIsInit, val => val && initCheckedInfo(), {
    immediate: true
  });
  /** 设置初次选中的值以及初始值 */
  function initCheckedInfo() {
    const {
      backfill: BACKFILL,
      field: FIELD,
      fields: FIELDS
    } = props;
    if (BACKFILL) {
      // 存在回填值且回填值中对应字段为真
      // 则设置回填值并不处理后续逻辑
      if (FIELDS) {
        const r = FIELDS.reduce((p, v) => {
          BACKFILL[v] && p.push(BACKFILL[v]);
          return p;
        }, []);
        if (r.length) {
          checked.value = r;
          option.updateWrapperQuery();
          return;
        }
      } else if (BACKFILL[FIELD]) {
        checked.value = insideGetChained(BACKFILL[FIELD]);
        option.updateWrapperQuery();
        return;
      }
    }
    // 不存在回填值, 看是否存在初始值
    // 存在设置默认值
    if (initialValue.value?.length) {
      // TODO 需考虑 emitPath 和多选的情况
      // 此种情况比较多 可封装成一个函数处理
      checked.value = typeof initialValue.value === 'string' ? insideGetChained(initialValue.value) : initialValue.value.slice();
      typeof initialValue.value === 'string' && (initialValue.value = clone(checked.value));
      option.updateWrapperQuery();
    }
  }
  // 提交字段发生改变时, 删除原有字段并更新最新值
  unwatchs.push(watch(() => props.fields || [props.field], (val, oldVal) => {
    val.toString() !== oldVal.toString() && wrapper && oldVal.forEach(o => val.includes(o) || wrapper.removeUnreferencedField(o));
    option.updateWrapperQuery();
  }));
  // 实时值发生变化时触发更新 - 共享同一个字段
  unwatchs.push(watch([() => props.fields?.toString() || props.field, () => props.fields?.map(v => props.query[v]).filter(Boolean) || props.query[props.field]], ([_field, val], [__field, oldVal]) => {
    // 仅在值发生变化时同步
    if (!realtimeFlag.value) return;
    if (_field !== __field || isEqualExcludeEmptyValue(val, oldVal)) return;
    // TODO 需考虑 emitPath 和多选的情况
    // checked.value = typeof val === 'string' ? insideGetChained(val) : val;
  }));
  // 回填值发生变化时触发更新
  unwatchs.push(watch(() => props.fields?.length ? props.fields.reduce((p, k) => {
    props.backfill?.[k] && p.push(props.backfill[k]);
    return p;
  }, []) : props.backfill?.[props.field], (value, oldVal) => {
    if (!sourceIsInit.value) return;
    if (isEqualExcludeEmptyValue(value, oldVal)) return;
    updateBackfillFlag();
    // TODO 判断最后一个值是否与选中项最后一个一致(多选需遍历)
    // 如果一致则不更新, 不一致, 则通过 insideGetChained 获取到树形再赋值
    // updateCheckedValue(value);
  }));
  // 存在依赖项
  unwatchs.push(watch([() => props.depend, () => props.dependFields, () => props.dependFields && [].concat(props.dependFields).map(k => props.query?.[k])], ([_depend, _dependFields], [__depend, __dependFields]) => {
    if (!realtimeFlag.value) return;
    // 是否启用依赖, 相同时启用才走后续逻辑, 不同时直接走后续逻辑
    if (_depend === __depend && !_depend) return;
    getOption('depend');
    // 类空值时, 不触发 change 事件
    // 防止表单类监测值发生改变时触发校验
    if (isEmptyValue(checked.value)) return;
    change(undefined);
  }));
  unwatchs.push(watch(() => props.getOptions, getOption.bind(null, 'initial'), {
    immediate: true
  }));
  /** 获取数据源发生变化事件 */
  function getOption(trigger) {
    props.getOptions?.(data => {
      remoteOption.value = data || [];
      sourceIsInit.value = true;
    }, props.query || {}, {
      trigger,
      options: toRaw(wrapper?.options) || {},
      changeDefaultValue(value) {
        initialValue.value = value;
        return this;
      },
      change(value, isInitial) {
        isInitial && (initialValue.value = value);
        change(value);
        return this;
      },
      search(value, isInitial) {
        isInitial && (initialValue.value = value);
        updateCheckedValue(value);
        wrapper?.search();
        return this;
      }
    });
  }
  /**
   * 更新选中值(父级也同步更改)
   * @param {ValueType | ValueType[]} values 待更改的值
   */
  function updateCheckedValue(values) {
    if (isEqualExcludeEmptyValue(checked.value, values)) return;
    checked.value = values;
    option.updateWrapperQuery();
  }
  /**
   * change 事件
   * @param {Array} values 待更改的值
   */
  function change(values) {
    updateCheckedValue(values);
    wrapper?.insetSearch();
  }
  /**
   * 根据提供的值获取其祖先(包括自身)
   */
  function insideGetChained(val) {
    if (isEmptyValue(val)) return [];
    const {
      valueKey,
      childrenKey
    } = props;
    return getChained(finalOption.value, item => item[valueKey] === val).map(v => v[valueKey], childrenKey).filter(Boolean);
  }
  return {
    wrapper,
    option,
    checked,
    getQuery,
    finalOption,
    insetDisabled,
    insetHide,
    change,
    reset: option.reset
  };
}

/** 树形条件类 props */
const treeProps = {
  ...commonProps,
  /** 不同层级返回不同的字段(可能存在的字段, 不传初始不会回填数据) */
  fields: {
    type: [Array]
  },
  /** 提交给后端的字段 */
  valueKey: {
    type: String,
    required: true
  },
  /** 子级键名 - 默认 children */
  childrenKey: {
    type: String
  },
  /** 是否返回全路径 */
  emitPath: {
    type: [Boolean],
    default: false
  },
  /** 下拉选项的数据源 */
  options: {
    type: Array,
    default: () => []
  },
  /** 获取数据源 */
  getOptions: {
    type: Function
  }
};

/** 封装 wrapper 组件必备的信息 */
function useWrapper(props, option) {
  const child = [];
  onBeforeUnmount(() => child.splice(0));
  const emptyValue = computed(() => props.emptyValue?.());
  /** 是否标记更新的字段, 防止卸载后的空字段占位 */
  let isLogField = false;
  let logFields = [];
  /** 记录所有条件的 options */
  const optionsObj = reactive({});
  /** 提供给子条件组件的方法 */
  const wrapperInstance = defineProvideValue({
    realtime: toRef(props, 'realtime', false),
    queryChangedInWrapper: ref(false),
    register(compOption) {
      child.push(compOption);
      const unregister = () => {
        isLogField = true;
        compOption.reset();
        compOption.updateWrapperQuery();
        const idx = child.indexOf(compOption);
        idx !== -1 && child.splice(idx, 1);
        props.searchAtDatumChanged && search();
        // TODO 不确定的一点, 数据源更改后是否需要重置整个数据
        // 如果需要重置, 得更新后第一次搜索事件时传递的搜索值
        isLogField = false;
        logFields.forEach(k => {
          del(query.value, k);
        });
        logFields = [];
      };
      const childInstance = getCurrentInstance();
      // vue2.7 实例挂载在 proxy 上, 内部逻辑取的 proxy 上的值
      // 虽然 @vue/composition-api 实例挂载在 proxy 上
      // 但是内部逻辑取的是整个 getCurrentInstance
      // @ts-expect-error vue2.7中取proxy属性
      childInstance && onBeforeUnmount(unregister, IS_COMPOSITION_VERSION ? childInstance.proxy : childInstance);
      return unregister;
    },
    updateQueryValue(field, value, nativeField) {
      props.emptyValues.includes(value) && value !== emptyValue.value && (value = emptyValue.value);
      if (isLogField) logFields.push(field);
      set(query.value, field, value);
      option?.fieldChange?.({
        field,
        value,
        query: query.value,
        nativeField
      });
    },
    insetSearch() {
      props.realtime && search();
    },
    search,
    removeUnreferencedField(field) {
      let sameFieldCount = 0;
      child.some(v => {
        hasOwn(v.getQuery(), field) && (sameFieldCount += 1);
        return sameFieldCount;
      });
      if (!sameFieldCount) {
        del(query.value, field);
      }
    },
    options: optionsObj
  });
  provide(provideKey, wrapperInstance);
  /** 内部条件最新的值, 在没触发搜索按钮前, 不会同步到外部 */
  const query = ref({
    ...props.backfill
  });
  // 由于 watch 主动处理了 backfill
  // 所以覆盖就没有存在的必要了
  // const getQuery = () => ({ ...query.value, ...props.backfill, ...changedQueryObj });
  const getQuery = () => ({
    ...query.value
  });
  watch(() => ({
    ...props.backfill
  }), (val, oldVal) => {
    wrapperInstance.queryChangedInWrapper.value = true;
    // 手动处理 query 的值于 backfill 保持一致
    // 防止 query.value 对象改变导致内部监听误触发
    Object.keys(query.value).forEach(k => {
      val && hasOwn(val, k) || delete query.value[k];
    });
    // #fix 只合并有变化的字段, 防止子级 watch 监听时误触发
    const newQuery = {};
    let isChanged = false;
    Object.keys(val).forEach(k => {
      if (val[k] !== oldVal[k]) {
        newQuery[k] = val[k];
        isChanged = true;
      }
    });
    isChanged && Object.assign(query.value, newQuery);
    isChanged ? nextTick(restoreQueryChanged) : restoreQueryChanged();
  }, {
    deep: true
  });
  function restoreQueryChanged() {
    wrapperInstance.queryChangedInWrapper.value = false;
  }
  async function search() {
    const msg = await validate();
    msg ? props.toast?.(msg) : option?.search?.(getQuery());
  }
  /** 重置所有条件的值 */
  function reset() {
    child.forEach(v => {
      v.reset();
      v.updateWrapperQuery();
    });
    option?.reset?.(getQuery());
  }
  /**
   * 重置为默认值
   * @param {boolean} [allowEmptyValue] 当默认值为空值时, 是否允许重置为空值
   */
  function resetField(allowEmptyValue) {
    child.forEach(v => {
      v.resetField(allowEmptyValue) && v.updateWrapperQuery();
    });
    option?.search?.(getQuery());
  }
  /** 自定义校验条件的值 */
  async function validate() {
    const r = await Promise.all(child.map(v => v.validator?.(query.value)));
    return r.find(v => v && typeof v === 'string');
  }
  return {
    child,
    wrapperInstance,
    query,
    getQuery,
    search,
    reset,
    resetField,
    validate
  };
}

/** 条件容器 props */
const wrapperProps = {
  /** 是否在数据发生变动后实时触发搜索事件 */
  realtime: {
    type: Boolean,
    default: undefined
  },
  /** 是否在数据源发生改变后触发一次搜索事件 */
  searchAtDatumChanged: {
    type: Boolean,
    default: undefined
  },
  /** 回填信息 */
  backfill: {
    type: Object
  },
  /** 校验失败时产生的提示 */
  toast: {
    type: Function,
    default: undefined
  },
  /** 配置空值 */
  emptyValues: {
    type: Array,
    default: () => ['', null, undefined]
  },
  /** 设置空值时返回的值 */
  emptyValue: {
    type: Function,
    default: undefined
  }
};

export { _interface as CoreCondition, IS_COMPOSITION_VERSION, clone, commonProps, defineCommonMethod, defineProvideValue, emptyToValue, get, getChained, getNode, hasOwn, isEmptyValue, isEqual, isEqualExcludeEmptyValue, isPrimitive, plainProps, provideKey, treeProps, usePlain, useTree, useWrapper, wrapperProps };
//# sourceMappingURL=index.esm.js.map
