import * as vue_demi from 'vue-demi';
import { Ref, VNode, PropType, WatchOptions, ExtractPropTypes, UnwrapRef } from 'vue-demi';

/** 判断是否是 2.7.* 版本, 监听生命周期需对该版本做处理 */
declare const IS_COMPOSITION_VERSION: boolean;
/** 容器注入的 key */
declare const provideKey = "condition-wrapper";
/** 容器注入值的类型 */
interface ProvideValue {
    /**
     * 是否实时触发
     */
    realtime: Ref<boolean | undefined>;
    /**
     * 当 query 变化是, 是否在 wrapper 处触发的
     */
    queryChangedInWrapper: Ref<boolean | undefined>;
    /**
     * 子组件需主动注册组件, 否则不会生效
     * @param {CommonMethod} config 提供父组件校验, 重置等方法
     *
     * @returns {() => void} 取消注册 - 默认会自动取消, 如果是异步任务内注册, 需自己手动取消
     */
    register: (config: CommonMethod) => () => void;
    /**
     * 子组件通知父级更新 query 中的值 - 静默修改, 不触发搜索事件
     * @param {string} field 更新的字段
     * @param {*} value 更新的值
     * @param {string} nativeField 原始提供的字段(不受 as, fields 等属性的影响)
     */
    updateQueryValue: (field: string, value: any, nativeField: string) => void;
    /**
     * 子组件内部值发生了变动, 由父级决定是否触发搜索事件(实时搜索时需要区分这两种方式)
     */
    insetSearch: () => void;
    /**
     * 提供给组件内部的直接触发到外部的搜索事件
     */
    search: () => Promise<string | void>;
    /** 删除内部无引用的字段 */
    removeUnreferencedField: (field: string) => void;
    /** 所有条件的 options 数据 */
    options: Record<string, any[]>;
}
declare function defineProvideValue<T extends ProvideValue>(option: T): T;
/** 子组件需暴露出来的公共属性 */
interface CommonMethod {
    /** 重置 */
    reset: () => void;
    /**
     * 重置为默认值
     * @param {boolean} [allowEmptyValue] 当默认值为空值时, 是否允许重置为空值
     */
    resetField: (allowEmptyValue?: boolean) => boolean;
    /** 更新父级中 query 的值 */
    updateWrapperQuery: () => void;
    /** 校验方法 */
    validator?: (query: Record<string, string>) => Promise<any> | any;
    /** 获取该组件拼接的参数 */
    getQuery: () => Record<string, any>;
}
declare function defineCommonMethod<T extends CommonMethod>(option: T): T;

/**
 * 空值转为提供的默认值
 * @param {*} val 值为空时转为默认值
 * @param {*} defaultVal
 */
declare function emptyToValue<T>(val: any, defaultVal: T): any;
/** 判断两个值是否相等, 排除掉空值 */
declare function isEqualExcludeEmptyValue(x: any, y: any): boolean;
/** 判断是否是空值(null, undefined, '') */
declare function isEmptyValue(val: any): boolean;
/** 判断两个值是否一致 */
declare function isEqual(x: any, y: any): boolean;
/** 判断是否是原始类型(number , string , boolean , symbol, bigint, undefined, null) */
declare function isPrimitive(value: any): boolean;
/** 拷贝 */
declare function clone<T>(obj: T, deep?: boolean): T;
/**
 * 获取指定层级的父级(包括自身)
 * @param {Record<string, any>[]} data 数据源
 * @param {(item) => boolean} cb 当前数据项是否匹配
 */
declare function getChained<T extends Record<string, any>>(data: T[], cb: (item: T) => boolean, childrenKey?: string): T[];
/**
 * 获取渲染节点
 * @param {string | object | Function} node 需渲染元素
 */
declare function getNode(node: string | ((...args: any[]) => VNode) | VNode | undefined | null, ...args: any[]): {};
/**
 * 通过字符串路径获取值
 * @example get(person, 'friends[0].name')
 */
declare function get<TDefault = unknown>(value: any, path: string, defaultValue?: TDefault): TDefault;
/**
 * 判断对象是否存在指定属性
 * @example hasOwn({}, 'a')
 */
declare function hasOwn<T extends Record<string, any>, K extends keyof T = keyof T>(obj: T, key: K): boolean;

/** 隐藏元素 */
type HideOption = boolean | ((query: {
    /** 实时 query */
    query: Record<string, any>;
    /** 回填值 query */
    backfill?: Record<string, any>;
    /** 当前组件暴露给父级的选项 */
    option: CommonMethod;
}) => any);
/** 获取远程数据源 */
interface GetOptions {
    (
    /** 数据执行后的回调 */
    cb: (data: Record<string, any>[]) => void, 
    /** 当前 query 对象 */
    query: Record<string, any>, 
    /** 额外的配置项 */
    option: TriggerOption): any;
}
/** 条件值可能的类型 */
type ValueType$2 = number | string | boolean | any[] | Record<string, any>;
/** 改变当前条件值触发方式 */
interface TriggerOption {
    /**
     * 触发来源
     * @enum {('initial'|'depend')} initial(初始化), depend(依赖项改变)
     */
    trigger: string;
    /**
     * 所有条件的数据源
     * @enum {Record<string, Record<string, any>[]>}
     */
    options: Record<string, Record<string, any>[]>;
    /**
     * 仅更改默认值
     * @param {*} value 需改变的值
     */
    changeDefaultValue: (value: any) => this;
    /**
     * 仅改变内部的值, 不触发搜索事件
     * @param {*} value 需改变的值
     * @param {boolean} [updateInitialValue] 是否将该值作为初始值(重置时使用)
     */
    change: (value: any, updateInitialValue?: boolean) => this;
    /**
     * 触发搜索事件
     * @param {*} value 需改变的值
     * @param {boolean} [updateInitialValue] 是否将该值作为初始值(重置时使用)
     */
    search: (value: any, updateInitialValue?: boolean) => this;
}
/** 自定义返回字段 */
type GetQuery = (value: any | any[], emptyToValue: typeof emptyToValue, props: Record<string, any>) => Record<string, any>;
/** 公共 props */
declare const commonProps: {
    /** 提交的字段 */
    readonly field: {
        readonly type: PropType<string>;
        readonly required: true;
    };
    /** 是否多选 */
    readonly multiple: {
        readonly type: PropType<boolean>;
        readonly default: undefined;
    };
    /** 当前条件对象 - 实时变化 */
    readonly query: {
        readonly type: PropType<Record<string, any>>;
        readonly required: true;
    };
    /** 回填值的对象 - 非实时变化 */
    readonly backfill: {
        readonly type: PropType<Record<string, any>>;
    };
    /** 禁用状态 */
    readonly disabled: {
        readonly type: PropType<HideOption>;
    };
    /** 是否隐藏 -> 如果是函数, 需传递依赖项, 可根据依赖项动态隐藏 */
    readonly hide: {
        readonly type: PropType<HideOption>;
    };
    /** 是否依赖其它字段 */
    readonly depend: {
        readonly type: PropType<boolean>;
    };
    /** 依赖字段发生变化后是否重置值 */
    readonly resetByDependValueChange: {
        readonly type: PropType<boolean>;
        readonly default: true;
    };
    /** 依赖字段 */
    readonly dependFields: {
        readonly type: PropType<string | string[]>;
    };
    /** 依赖字段监听选项 */
    readonly dependWatchOption: {
        readonly type: PropType<WatchOptions<boolean>>;
    };
    /** 是否依赖其它字段的数据源 - 数据发生变动时触发 getOptions */
    readonly optionsDepend: {
        readonly type: PropType<boolean>;
    };
    /** 数据源依赖字段 - 不传取 dependFields */
    readonly optionsDependFields: {
        readonly type: PropType<string | string[]>;
    };
    /** 重置时是否置为初始值 */
    readonly resetToInitialValue: {
        readonly type: PropType<boolean>;
    };
    /** 空置时提交的值 - 默认置为 undefined */
    readonly emptyValue: {
        readonly type: PropType<string | number | boolean | null | undefined>;
        readonly default: undefined;
    };
    /** 校验函数, 返回字符串不通过, 会触发提示 - 提交时触发 */
    readonly validator: {
        readonly type: PropType<((query: Record<string, any>) => any) | ((query: Record<string, any>) => Promise<any>)>;
    };
    /** 自定义返回字段 */
    readonly customGetQuery: {
        readonly type: PropType<GetQuery>;
    };
    /** 设置默认值 */
    readonly defaultValue: {
        readonly type: PropType<ValueType$2 | ((query: Record<string, any>, backfill?: Record<string, any>) => ValueType$2)>;
        readonly default: undefined;
    };
};

/** 扁平条件类 props */
declare const plainProps: {
    /** 字段集(多选时, 每个下标对应的字段可能不一样) */
    readonly fields: {
        readonly type: PropType<string[]>;
    };
    /** 多字段时转换成选中值 */
    readonly backfillToValue: {
        readonly type: PropType<(values: string | string[], fields: string | string[], backfill?: Record<string, any>) => string | string[]>;
        readonly default: (v: any) => any;
    };
    /** 数据源 */
    readonly options: {
        readonly type: PropType<Record<string, any>[]>;
        readonly default: () => never[];
    };
    /** 动态获取数据源 */
    readonly getOptions: {
        readonly type: PropType<GetOptions>;
    };
    readonly field: {
        readonly type: PropType<string>;
        readonly required: true;
    };
    readonly multiple: {
        readonly type: PropType<boolean>;
        readonly default: undefined;
    };
    readonly query: {
        readonly type: PropType<Record<string, any>>;
        readonly required: true;
    };
    readonly backfill: {
        readonly type: PropType<Record<string, any>>;
    };
    readonly disabled: {
        readonly type: PropType<HideOption>;
    };
    readonly hide: {
        readonly type: PropType<HideOption>;
    };
    readonly depend: {
        readonly type: PropType<boolean>;
    };
    readonly resetByDependValueChange: {
        readonly type: PropType<boolean>;
        readonly default: true;
    };
    readonly dependFields: {
        readonly type: PropType<string | string[]>;
    };
    readonly dependWatchOption: {
        readonly type: PropType<vue_demi.WatchOptions<boolean>>;
    };
    readonly optionsDepend: {
        readonly type: PropType<boolean>;
    };
    readonly optionsDependFields: {
        readonly type: PropType<string | string[]>;
    };
    readonly resetToInitialValue: {
        readonly type: PropType<boolean>;
    };
    readonly emptyValue: {
        readonly type: PropType<string | number | boolean | null | undefined>;
        readonly default: undefined;
    };
    readonly validator: {
        readonly type: PropType<((query: Record<string, any>) => any) | ((query: Record<string, any>) => Promise<any>)>;
    };
    readonly customGetQuery: {
        readonly type: PropType<GetQuery>;
    };
    readonly defaultValue: {
        readonly type: PropType<ValueType$2 | ((query: Record<string, any>, backfill?: Record<string, any> | undefined) => ValueType$2)>;
        readonly default: undefined;
    };
};

/** 外部需传递的 props */
type PlainProps$1 = ExtractPropTypes<typeof plainProps>;
type ValueType$1 = string | number | boolean | null | undefined | Record<string, any>;
/** 封装扁平组件必备的信息 */
declare function usePlain(props: PlainProps$1): {
    wrapper: ProvideValue | undefined;
    option: {
        reset(this: void): void;
        resetField(this: void, allowEmptyValue?: boolean): boolean;
        updateWrapperQuery(this: void): void;
        readonly validator: ((query: Record<string, any>) => any) | ((query: Record<string, any>) => Promise<any>) | undefined;
        getQuery: () => Record<string, any>;
    };
    checked: vue_demi.Ref<ValueType$1 | ValueType$1[]>;
    getQuery: () => Record<string, any>;
    insetDisabled: vue_demi.Ref<boolean>;
    insetHide: vue_demi.Ref<boolean>;
    finalOption: vue_demi.ComputedRef<Record<string, any>[]>;
    updateCheckedValue: (value: ValueType$1 | ValueType$1[]) => void;
    change: (value: ValueType$1 | ValueType$1[]) => void;
    reset: (this: void) => void;
};

/** 树形条件类 props */
declare const treeProps: {
    /** 不同层级返回不同的字段(可能存在的字段, 不传初始不会回填数据) */
    readonly fields: {
        readonly type: PropType<string[]>;
    };
    /** 提交给后端的字段 */
    readonly valueKey: {
        readonly type: PropType<string>;
        readonly required: true;
    };
    /** 子级键名 - 默认 children */
    readonly childrenKey: {
        readonly type: PropType<string>;
    };
    /** 是否返回全路径 */
    readonly emitPath: {
        readonly type: PropType<boolean>;
        readonly default: false;
    };
    /** 下拉选项的数据源 */
    readonly options: {
        readonly type: PropType<Record<string, any>[]>;
        readonly default: () => never[];
    };
    /** 获取数据源 */
    readonly getOptions: {
        readonly type: PropType<GetOptions>;
    };
    readonly field: {
        readonly type: PropType<string>;
        readonly required: true;
    };
    readonly multiple: {
        readonly type: PropType<boolean>;
        readonly default: undefined;
    };
    readonly query: {
        readonly type: PropType<Record<string, any>>;
        readonly required: true;
    };
    readonly backfill: {
        readonly type: PropType<Record<string, any>>;
    };
    readonly disabled: {
        readonly type: PropType<HideOption>;
    };
    readonly hide: {
        readonly type: PropType<HideOption>;
    };
    readonly depend: {
        readonly type: PropType<boolean>;
    };
    readonly resetByDependValueChange: {
        readonly type: PropType<boolean>;
        readonly default: true;
    };
    readonly dependFields: {
        readonly type: PropType<string | string[]>;
    };
    readonly dependWatchOption: {
        readonly type: PropType<vue_demi.WatchOptions<boolean>>;
    };
    readonly optionsDepend: {
        readonly type: PropType<boolean>;
    };
    readonly optionsDependFields: {
        readonly type: PropType<string | string[]>;
    };
    readonly resetToInitialValue: {
        readonly type: PropType<boolean>;
    };
    readonly emptyValue: {
        readonly type: PropType<string | number | boolean | null | undefined>;
        readonly default: undefined;
    };
    readonly validator: {
        readonly type: PropType<((query: Record<string, any>) => any) | ((query: Record<string, any>) => Promise<any>)>;
    };
    readonly customGetQuery: {
        readonly type: PropType<GetQuery>;
    };
    readonly defaultValue: {
        readonly type: PropType<ValueType$2 | ((query: Record<string, any>, backfill?: Record<string, any> | undefined) => ValueType$2)>;
        readonly default: undefined;
    };
};

/** 选中值类型 */
type ValueType = string | number | boolean | Record<string, any>;
/** 外部需传递的 props */
type TreeProps$1 = ExtractPropTypes<typeof treeProps>;
/**
 * @file 目前该组件尚未被使用
 */
/** 封装 tree 组件必备的信息 */
declare function useTree(props: TreeProps$1): {
    wrapper: ProvideValue | undefined;
    option: {
        reset(this: void): void;
        resetField(this: void, allowEmptyValue?: boolean): boolean;
        readonly validator: ((query: Record<string, any>) => any) | ((query: Record<string, any>) => Promise<any>) | undefined;
        updateWrapperQuery(): void;
        getQuery: () => Record<string, any>;
    };
    checked: vue_demi.Ref<ValueType[] | ValueType[][] | undefined>;
    getQuery: () => Record<string, any>;
    finalOption: vue_demi.ComputedRef<Record<string, any>[]>;
    insetDisabled: vue_demi.Ref<boolean>;
    insetHide: vue_demi.Ref<boolean>;
    change: (values?: ValueType[] | ValueType[][]) => void;
    reset: (this: void) => void;
};

/** 条件容器 props */
declare const wrapperProps: {
    /** 是否在数据发生变动后实时触发搜索事件 */
    readonly realtime: {
        readonly type: PropType<boolean>;
        readonly default: undefined;
    };
    /** 是否在数据源发生改变后触发一次搜索事件 */
    readonly searchAtDatumChanged: {
        readonly type: PropType<boolean>;
        readonly default: undefined;
    };
    /** 回填信息 */
    readonly backfill: {
        readonly type: PropType<Record<string, any>>;
    };
    /** 校验失败时产生的提示 */
    readonly toast: {
        readonly type: PropType<(msg: string) => void>;
        readonly default: undefined;
    };
    /** 配置空值 */
    readonly emptyValues: {
        readonly type: PropType<any[]>;
        readonly default: () => (string | null | undefined)[];
    };
    /** 设置空值时返回的值 */
    readonly emptyValue: {
        readonly type: PropType<() => any>;
        readonly default: undefined;
    };
};

/** 外部需传递的 props */
type WrapperProps$1 = ExtractPropTypes<typeof wrapperProps>;
/** 外部传递的方法 */
interface WrapperOption {
    /** 触发搜索事件 */
    search?: (params: Record<string, any>) => void;
    /** 触发重置事件 */
    reset?: (params: Record<string, any>) => void;
    /**
     * 字段值发生改变时触发
     * @param {object} option 提供的
     * @param {string} option.field 实际改变的键
     * @param {*} option.value
     * @param {object} option.query
     * @param {string} option.nativeField 原始健(不受 as, fields 等属性影响)
     */
    fieldChange?: (option: {
        field: string;
        value: any;
        query: Record<string, any>;
        nativeField: string;
    }) => void;
}
/** 封装 wrapper 组件必备的信息 */
declare function useWrapper(props: WrapperProps$1, option?: WrapperOption): {
    child: CommonMethod[];
    wrapperInstance: {
        realtime: vue_demi.Ref<boolean>;
        queryChangedInWrapper: vue_demi.Ref<boolean>;
        register(compOption: CommonMethod): () => void;
        updateQueryValue(field: string, value: any, nativeField: string): void;
        insetSearch(): void;
        search: () => Promise<void>;
        removeUnreferencedField(field: string): void;
        options: Record<string, any>;
    };
    query: vue_demi.Ref<Record<string, string>>;
    getQuery: () => {
        [x: string]: string;
    };
    search: () => Promise<void>;
    reset: () => void;
    resetField: (allowEmptyValue?: boolean) => void;
    validate: () => Promise<string>;
};

type BuiltInField = 'field' | 'query';
/** 将值改为原始值或引用值 */
type MaybeRef<T> = T | Ref<T>;
/** 将整个集合改为原始值或引用值 */
type DeepMaybeRef<T> = T extends Ref<infer V> ? MaybeRef<V> : T extends (...args: any) => any ? MaybeRef<T> : T extends Array<any> | Record<string, any> ? {
    [K in keyof T]: DeepMaybeRef<T[K]>;
} : MaybeRef<T>;
/** 获取实际值(去除 ref 引用) */
type ToRaw<T> = UnwrapRef<T>;
interface WrapperProps extends Omit<ExtractPropTypes<OmitDefaultKey<typeof wrapperProps>>, BuiltInField> {
}
interface PlainProps extends Omit<ExtractPropTypes<OmitDefaultKey<typeof plainProps>>, BuiltInField> {
}
interface TreeProps extends Omit<ExtractPropTypes<OmitDefaultKey<typeof treeProps>>, BuiltInField> {
}
type OmitDefaultKey<T> = T extends Record<string | symbol, any> ? {
    [K in keyof T]: T[K] extends {
        default: any;
    } ? Omit<T[K], 'default'> : T[K];
} : T;

type _interface_BuiltInField = BuiltInField;
type _interface_DeepMaybeRef<T> = DeepMaybeRef<T>;
type _interface_MaybeRef<T> = MaybeRef<T>;
type _interface_OmitDefaultKey<T> = OmitDefaultKey<T>;
type _interface_PlainProps = PlainProps;
type _interface_ToRaw<T> = ToRaw<T>;
type _interface_TreeProps = TreeProps;
type _interface_WrapperProps = WrapperProps;
declare namespace _interface {
  export {
    _interface_BuiltInField as BuiltInField,
    _interface_DeepMaybeRef as DeepMaybeRef,
    _interface_MaybeRef as MaybeRef,
    _interface_OmitDefaultKey as OmitDefaultKey,
    _interface_PlainProps as PlainProps,
    _interface_ToRaw as ToRaw,
    _interface_TreeProps as TreeProps,
    _interface_WrapperProps as WrapperProps,
  };
}

export { CommonMethod, _interface as CoreCondition, GetOptions, GetQuery, HideOption, IS_COMPOSITION_VERSION, PlainProps$1 as PlainProps, ProvideValue, TreeProps$1 as TreeProps, TriggerOption, ValueType$2 as ValueType, WrapperOption, WrapperProps$1 as WrapperProps, clone, commonProps, defineCommonMethod, defineProvideValue, emptyToValue, get, getChained, getNode, hasOwn, isEmptyValue, isEqual, isEqualExcludeEmptyValue, isPrimitive, plainProps, provideKey, treeProps, usePlain, useTree, useWrapper, wrapperProps };
